"""
module: slideshowfolder
created by: Johnpaul Burbank <jpburbank@tegus.ca>
Date: July 8, 2007
"""

#Zope / Z3
from zope.interface import implements, providedBy, alsoProvides
from zope.interface import directlyProvides, directlyProvidedBy
from Products.Five.browser import BrowserView
from Acquisition import aq_inner

#Plone
from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.interface.topic import IATTopic

#Project
from Products.slideshowfolder.interfaces import ISlideShowView, IFolderSlideShowView, ISlideShowFolder
from Products.slideshowfolder.slideshowsetting import SlideShowSetting
from Products.slideshowfolder.config import TRANSITIONS
from Products.slideshowfolder.config import PROJ
from Products.slideshowfolder import HAS_PLONE30

try:
    #For Zope 2.10.4
    from zope.annotation.interfaces import IAttributeAnnotatable
    from zope.annotation.interfaces import IAnnotations
except ImportError:
    #For Zope 2.9
    from zope.app.annotation.interfaces import IAttributeAnnotatable
    from zope.app.annotation.interfaces import IAnnotations

if HAS_PLONE30:
    # Images don't have a workflow by default.
    WF_FILTER = None
else:
    WF_FILTER = 'published'


class SlideShowFolderView(BrowserView):
    implements(ISlideShowView)

    __catalog_args = dict(Type='Image', sort_on='getObjPositionInParent')

    def slideTransition(self):
        """Returns type of slide transition"""
        settings = SlideShowSetting(self.context)
        return settings.slideTransition

    def slideDuration(self):
        """Returns the time in second before the next slide transition"""
        settings = SlideShowSetting(self.context)
        return settings.slideDuration

    def panAmount(self):
        """Returns the amount of pan to use in a pan or combo transition"""
        settings = SlideShowSetting(self.context)
        return settings.panAmount

    def zoomAmount(self):
        """Returns the amount of zoom to use in a zoom or combo transition"""
        settings = SlideShowSetting(self.context)
        return settings.zoomAmount

    def transitionTime(self):
        """Returns the time in second for a slide transition to take place"""
        settings = SlideShowSetting(self.context)
        return settings.transitionTime

    def showHeight(self):
        """Returns the height of the slideshow"""
        settings = SlideShowSetting(self.context)
        return settings.showHeight

    def showWidth(self):
        """Returns the width of the slideshow"""
        settings = SlideShowSetting(self.context)
        return settings.showWidth

    def thumbnails(self):
        """Returns a boolean whether thumbnail mode has been chosen"""
        settings = SlideShowSetting(self.context)
        return settings.thumbnails

    def fast(self):
        """Returns a boolean whether arrow fast has been chosen"""
        settings = SlideShowSetting(self.context)
        return settings.fast

    def arrows(self):
        """Returns a boolean whether arrow mode has been chosen"""
        settings = SlideShowSetting(self.context)
        return settings.arrows


    def fields(self):
        """Returns the fields for this view"""
        trans = {}
        trans['label'] = 'Slide Transition'
        trans['fieldName'] = 'slideTransition'
        trans['description'] = 'Transition between slides'
        trans['value'] = self.slideTransition
        trans['type'] = 'select'
        trans['options'] = TRANSITIONS
        dur = {}
        dur['label'] = 'Slide Duration'
        dur['description'] = 'Time in seconds between slide transitions'
        dur['fieldName'] = 'slideDuration'
        dur['value'] = self.slideDuration
        dur['type'] = 'text'
        pan = {}
        pan['label'] = 'Pan Amount'
        pan['fieldName'] = 'panAmount'
        pan['description'] = 'Amount to pan images.  Only used for pan and combo transitions.  Enter a value from 0 to 100 or rand for a random amount. You must set width smaller than the width of the smallest image for panning to work.'
        pan['value'] = self.panAmount
        pan['type'] = 'text'
        zoom = {}
        zoom['label'] = 'Zoom Amount'
        zoom['fieldName'] = 'zoomAmount'
        zoom['description'] = 'Amount to zoom images.  Only used for zoom and combo transitions.  Enter a value from 0 to 100 or rand for a random amount. Note that zoom effects can look a bit weird on Windows/IE.'
        zoom['value'] = self.zoomAmount
        zoom['type'] = 'text'
        transTime = {}
        transTime['label'] = 'Transition Time'
        transTime['description'] = 'Time in seconds for a slide transition to take place.'
        transTime['fieldName'] = 'transitionTime'
        transTime['value'] = self.transitionTime
        transTime['type'] = 'text'
        width = {}
        width['label'] = 'Slideshow Width'
        width['description'] = 'Width of the slideshow. Defaults to the width of the first image.'
        width['fieldName'] = 'showWidth'
        width['value'] = self.showWidth
        width['type'] = 'text'
        height = {}
        height['label'] = 'Slideshow Height'
        height['description'] = 'Height of the slideshow. Defaults to the height of the first image.'
        height['fieldName'] = 'showHeight'
        height['value'] = self.showHeight
        height['type'] = 'text'
        thumb = {}
        thumb['label'] = 'Thumbnail Navigation'
        thumb['description'] = 'Include navigation thumbnails below the image.'
        thumb['fieldName'] = 'thumbnails'
        thumb['value'] = self.thumbnails
        thumb['type'] = 'checkbox'
        arrow = {}
        arrow['label'] = 'Arrow Navigation'
        arrow['description'] = 'Include forward and back arrows in navigation below image.'
        arrow['fieldName'] = 'arrows'
        arrow['value'] = self.arrows
        arrow['type'] = 'checkbox'
        fast = {}
        fast['label'] = 'Fast Mode'
        fast['description'] = 'Skips the transition when a new thumbnail is selected.'
        fast['fieldName'] = 'fast'
        fast['value'] = self.fast
        fast['type'] = 'checkbox'
        return (trans, dur, pan, zoom, transTime, width, height, thumb, arrow, fast)

    def getTransitionTypes(self):
        """Returns a tuple of possible slide transitions"""
        return TRANSITIONS

    def _getSlideshowElements(self, wf_filter=WF_FILTER):
        """Convenience method to do the actual catalog calls.  Returns a list of brains or None"""
        # If we're dealing with something that looks like a Smart Folder/Collection,
        # use its criteria for getting images.  If not, we'll use our own.
        if IATTopic.providedBy(self.context):
            results = self.context.queryCatalog()
            # since we don't really know what the criteria were for the Topic, we
            # should filter it down now so we don't cause weird errors
            results = [brain for brain in results if brain.meta_type=='ATImage']
        # Not Topical, we're assuming it's otherwise folderish
        else:
            cat = getToolByName(self.context, 'portal_catalog')
            args = self.__catalog_args
            path = '/'.join(self.context.getPhysicalPath())
            args['path'] = {'query' : path, 'depth' : 1 }
            if wf_filter is not None:
                args['review_state'] = wf_filter
            results = cat(**args)

        return results
        

    def getSeedImage(self, wf_filter=WF_FILTER):
        """Return the parameters for the first image in the slide show.

        Returns a dict with values for:
            src
            width
            height
        """
        results = self._getSlideshowElements(wf_filter)
        if results:
            img = results[0].getObject()
            return {
                "src":    img.absolute_url(),
                "width":  img.width,
                "height": img.height,
                }
        else:
            return None

    def getSlideshowSettings(self, wf_filter=WF_FILTER):
        """Returns a list of settings for the current slideshowfolder in a dict."""

        results = self._getSlideshowElements(wf_filter)
        if results:
            names = ["%s/image_large" % item.getURL() for item in results]
            imageDescriptions = [item.Description.decode('UTF-8') for item in results]

            # Raw data from settings
            slideDurationSetting = self.slideDuration()
            transitionZoomAmount = self.zoomAmount()
            transitionTimeSetting = self.transitionTime()
            showHeightSetting = self.showHeight()
            showWidthSetting = self.showWidth()
            thumbnailsSetting = self.thumbnails()
            arrowsSetting = self.arrows()
            fastSetting = self.fast()

            #Building the javascript string
            options = {"strings": {},
                       "lists": {},
                       }
            options['strings']['type']  = self.slideTransition()
            options['strings']['pan'] = self.panAmount()
            options['strings']['zoom'] = transitionZoomAmount
            if slideDurationSetting or transitionTimeSetting:
                options['strings']['duration'] = '[%i, %i]' % (int(transitionTimeSetting)*1000, int(slideDurationSetting)*1000)

            seed_image = results[0].getObject()
            if showHeightSetting <= 0:
                showHeightSetting = seed_image.height
            options['strings']['height'] = showHeightSetting
            if showWidthSetting <= 0:
                showWidthSetting = seed_image.width
            options['strings']['width'] = showWidthSetting
            mode = ''
            if thumbnailsSetting:
                if arrowsSetting:
                    mode = "arrows+thumbnails"
                    if fastSetting:
                        mode = "arrows+thumbnails fast"
                elif fastSetting:
                    mode = "thumbnails fast"
                else:
                    mode = "thumbnails"
            elif arrowsSetting:
                if fastSetting:
                    mode = "arrows fast"
                else:
                    mode = "arrows"
            if mode:
                options['strings']['navigation'] = mode
            if imageDescriptions:
                options['lists']['captions'] = imageDescriptions
            options['lists']['images'] = names
            return options
        return None


class FolderSlideShowView(BrowserView):
    """Provides view methods in regards to the transition from folder to slideshow folder"""
    implements(IFolderSlideShowView)
    iprovides = (IAttributeAnnotatable, ISlideShowFolder)

    def isSlideshow(self):
        """Returns true if we're implementing the ISlideShowFolder interface.

        (Note: we're using implementation of that interface as a proxy for a few other
        behaviors that aren't technically w/in the scope of the interface, such
        as having the view selected and the config tab added.)"""
        return ISlideShowFolder.providedBy(self.context)

    def makeSlideshow(self):
        """Make a folderish object a slideshowfolder"""
        # two steps: provide interfaces, manually set "layout"
        for inter in self.iprovides:
            if not inter.providedBy(self.context):
                alsoProvides(self.context, inter)

        folder = aq_inner(self.context)
        folder.layout = 'folder_slideshow'
        plone_utils = getToolByName(self.context, 'plone_utils')
        plone_utils.addPortalMessage("This folder is now designated a slideshow.")
        if hasattr(self.request, 'RESPONSE'):
            self.request.RESPONSE.redirect(self.context.absolute_url())

    def unmakeSlideshow(self):
        """Remove all traces of slideshow-ness from a folder"""
        folder = aq_inner(self.context)
        # 1) stop providing the interface
        # AFAICT, there's no direct opposite of alsoProvides
        # I got the following technique from a IRC chat transcript:
        #           http://zope3.pov.lt/irclogs/%23zope3-dev.2007-03-19.log.html
        # (search for "IUCSubSite")
        if ISlideShowFolder in providedBy(folder):
            directlyProvides(folder, directlyProvidedBy(folder)-ISlideShowFolder)

        # 2) delete the "layout" attribute
        # This way, we'll fallback gracefully to whatever the current default view for folders is
        if hasattr(folder, 'layout'):
            delattr(folder, 'layout')

        # 3) we should delete the metadata saved in the annotation, too
        annotations = IAnnotations(self.context)
        if annotations.has_key(PROJ):
            del annotations[PROJ]

        if hasattr(self.request, 'RESPONSE'):
            plone_utils = getToolByName(self.context, 'plone_utils')
            plone_utils.addPortalMessage("Completely removed slideshow capabilities.")
            self.request.RESPONSE.redirect(self.context.absolute_url())
