"""
module: slideshowfolder
created by: Johnpaul Burbank <jpburbank@tegus.ca>
Date: July 8, 2007
"""

#Zope / Z3
from zope.interface import Interface
from zope import schema

#Project
from config import TRANSITIONS

class ISlideShowFolder(Interface):
    """Marker interface for a folder that holds content that implement IATImage"""
    


class ISlideShowSetting(Interface):
    """Slide show setting fields"""
    slideTransition = schema.TextLine(title=u'Transition type', default=TRANSITIONS[0])
    slideDuration = schema.Int(title=u'Slide Duration', default=5)
    panAmount = schema.TextLine(title=u'Pan Amount', default=u'rand')
    zoomAmount = schema.TextLine(title=u'Zoom Amount', default=u'rand')
    transitionTime = schema.Int(title=u'Transition Time', default=5)
    showWidth = schema.Int(title=u'Slideshow Width')
    showHeight = schema.Int(title=u'Slideshow Height')
    thumbnails = schema.Int(title=u'Thumbnail Mode')
    fast = schema.Bool(title=u'Fast Mode')
    arrows = schema.Bool(title=u'Arrows Mode')

    
class ISlideShowView(Interface):
    """Browser view for the slide show folder"""
        
    def slideTransition():
        """Returns type of slide transition"""
        
    def slideDuration():
        """Returns the time in second before the next slide transition"""
    
    def panAmount():
        """Returns the amount to pan images in pan or combo transitions"""

    def zoomAmount():
        """Returns the amount to zoom images in zoom or combo transitions"""
    
    def transitionTime():
        """Returns the time in second for a slide transition to take place"""
        
    def showHeight():
        """Returns the height of the slideshow"""
        
    def showWidth():
        """Returns the width of the slideshow"""
        
    def thumbnails():
        """Returns a boolean whether thumbnail mode has been chosen"""
        
    def fast():
        """Returns a boolean whether arrow fast has been chosen"""
        
    def arrows():
        """Returns a boolean whether arrow mode has been chosen"""
        
    def fields():
        """Returns the fields for this view"""
        
    def getTransitionTypes():
        """Returns a tuple of possible slide transitions"""
    
    def getSeedImage(wf_filter='published'):
        """Return the parameters for the first image in the slide show.
        
           Returns a dict with values for:
                src
                width
                height

           If wf_filter is None, the slideshow will not filter on workflow.  Otherwise, it will
           will filter on whatever value is passed in.
        """

    def getSlideshowSettings(wf_filter='published'):
        """Returns a list of settings for the current slideshowfolder in a dict.
           Keys: 'strings', 'lists'.
            
           If wf_filter is None, the slideshow will not filter on workflow.  Otherwise, it will
           will filter on whatever value is passed in.
        """
           
class IFolderSlideShowView(Interface):
    """Provides view methods in regards to the transition from folder to slideshow folder"""
    
    def isSlideshow():
        """Returns true if we're implementing the ISlideShowFolder interface.
        
        (Note: we're using implementation of that interface as a proxy for a few other
        behaviors that aren't technically w/in the scope of the interface, such
        as having the view selected and the config tab added.)"""

    # def canSlideshow():
    #     """Determines if a folderish type can become a slideshowfolder"""
        
    def makeSlideshow():
        """Make a folderish object a slideshowfolder"""
    
    def unmakeSlideshow():
        """Remove all traces of slideshow-ness from a folder"""
    