import os
from Products.CMFCore.utils import getToolByName

from base import SlideshowFolderTestCase
from Products.slideshowfolder import HAS_PLONE30
from Products.slideshowfolder.browser import FolderSlideShowView
from Products.slideshowfolder.browser import SlideShowFolderView

from Globals import package_home
PACKAGE_HOME = package_home(globals())

# borrowed from ATCT's test_atimage
def loadImage(name, size=0):
    """Load image from testing directory
    """
    path = os.path.join(PACKAGE_HOME, 'input', name)
    fd = open(path, 'rb')
    data = fd.read()
    fd.close()
    return data

TEST_IMAGE = loadImage('test.jpg')

class TestBrowserViews(SlideshowFolderTestCase):
    def afterSetUp(self):
        """Load a real image for use in tests"""
        self.setRoles(['Manager'])
        self.folder.invokeFactory('Folder', id="test_folder", title="Test Slideshow",)
        self.test_folder = getattr(self.folder, 'test_folder')
        self.test_folder.invokeFactory('Image', 'test.jpg')
        self.test_image = getattr(self.test_folder, 'test.jpg')
        self.test_image.setImage(TEST_IMAGE, mimetype='image/jpg', filename='test.jpg')
        self.test_image.setDescription(u'Qué linda esta flor')
        self.test_image.setTitle('Flower Power')
        if not HAS_PLONE30:
            workflow = getToolByName(self.portal, 'portal_workflow')
            workflow.doActionFor(self.test_image, 'publish')


    def testFolderAsSlideShow(self):
        view = FolderSlideShowView(self.test_folder, self.app.REQUEST)
        view.makeSlideshow()
        slideshow_view = SlideShowFolderView(self.test_folder, self.app.REQUEST)
        seed_image = slideshow_view.getSeedImage() 
        self.failUnless(seed_image['height'] == 480, "Height was %s instead of 480" % seed_image['height'])
        self.failUnless(seed_image['width'] == 640, "Width was %s instead of 480" % seed_image['width'])
        
        settings = slideshow_view.getSlideshowSettings()
        # we're implicitly testing the Unicode-ness of the description-as-caption
        self.failUnless(settings['lists']['captions'][0]==u'Qué linda esta flor',
                        "Image caption not right; was : %s" % settings['lists']['captions'][0])
        self.failUnless(settings['lists']['images'][0]=='http://nohost/plone/Members/test_user_1_/test_folder/test.jpg/image_large',
                        "Image url not right; was: %s" % settings['lists']['images'][0])

    def testTopicAsSlideShow(self):
        self.folder.invokeFactory('Topic', 'sf')
        topic = getattr(self.folder, 'sf')
        type_crit = topic.addCriterion('Type', 'ATPortalTypeCriterion')
        type_crit.setValue('Image')
        
        view = FolderSlideShowView(topic, self.app.REQUEST)
        view.makeSlideshow()
        slideshow_view = SlideShowFolderView(topic, self.app.REQUEST)
        seed_image = slideshow_view.getSeedImage() 
        self.failUnless(seed_image['height'] == 480, "Height was %s instead of 480" % seed_image['height'])
        self.failUnless(seed_image['width'] == 640, "Width was %s instead of 480" % seed_image['width'])
        
        settings = slideshow_view.getSlideshowSettings()

        self.failUnless(settings['lists']['captions'][0]==u'Qué linda esta flor',
                        "Image caption not right; was : %s" % settings['lists']['captions'][0])
        self.failUnless(settings['lists']['images'][0]=='http://nohost/plone/Members/test_user_1_/test_folder/test.jpg/image_large',
                        "Image url not right; was: %s" % settings['lists']['images'][0])

        

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBrowserViews))
    return suite
